% Line of Best Fit (linear regression) demonstration for
% lines of the form y = mx.
%
% This code generates N (x,y) samples based upon a positive or negative
% correlation + some random noise. A least-squares regression is then
% performed and graphs and Excel sheet(s) are generated.
%
% It is pre-assumed that y = 0 when x = 0 i.e. for y = mx and direct
% proportion. i.e. the regression line will pass through the origin.
function y_mx_fit_simulation

%% INPUTS %%

%Create random data based upon average line y = m*x
N = 42; m = 1.23;
xmin = 0; xmax = 10;

%Characterize normally distributed deviations from (x,y)
sigma_x = 2; sigma_y = 2;

%Fontsize for graphs
fsize = 18;

%%

%Create data vectors
x = linspace( xmin, xmax, N ) + sigma_x*randn(1,N);
y = m*x + sigma_y*randn(1,N);

%Determine line of best fit to x,y data
[yfit,xfit,r,m,dm,yupper,ylower,s] = bestfit(x,y);

%

%Determine filename
filename = ['bestfit y=mx simulation ',strrep(datestr(now),':','-')];

%Plot line of best fit, overlaid with data
figure('name','line of best fit','color',[1 1 1]);
plot( xfit, yfit, 'k-', xfit, yupper, 'b--', xfit, ylower, 'g--' );
hold on; plot( x,y,'r+');
xlabel('x','fontsize',fsize); ylabel('y','fontsize',fsize);
set(gca,'fontsize',fsize); grid on;
title( {['Line of best fit y = ( ',num2str(m),' +/- ',num2str(dm,3),...
    ' )x, r = ',num2str(r,3)] }, 'fontsize',fsize );
print( gcf, [filename,'.png'],'-dpng','-r300' );
close(gcf);

%

%Create Excel sheet with best-fit analysis pre-populated
X{1,1} = ['LINE OF BEST FIT y = mx DEMONSTRATION. Auto generated at ',datestr(now)];
X{2,12} = 'N'; X{2,13} = N;
X{5,1} = 'x'; X{5,2} = 'y'; X{5,3} = 'x^2'; X{5,4} = 'y^2'; X{5,5} = 'xy';
X{5,6} = 'xfit'; X{5,7} = 'yfit'; X{5,8} = 'ylower'; X{5,9} = 'yupper';
X{5,11} = 'xbar'; X{5,12} = 'ybar'; X{5,13} = 'x^2 bar'; X{5,14} = 'y^2 bar'; X{5,15} = 'xy bar'; 
X{5,17} = 'Vx'; X{5,18} = 'Vy'; X{5,19} = 'Cov[x,y]'; X{5,20} = 's';
X{8,11} = 'r'; X{8,12} = 'r^2';  X{8,14} = 'm';  X{8,17} = 'dm';
for n=1:length(x)
    X{5+n,1} = x(n);
    X{5+n,2} = y(n);
    X{5+n,3} = x(n)^2;
    X{5+n,4} = y(n)^2;
    X{5+n,5} = x(n)*y(n);
    X{5+n,6} = xfit(n);
    X{5+n,7} = yfit(n);
    X{5+n,8} = ylower(n);
    X{5+n,9} = yupper(n);
end
X{6,11} = mean(x);  X{6,12} = mean(y); X{6,13} = mean(x.^2);
X{6,14} = mean(y.^2); X{6,15} = mean(y.*x);
X{6,17} = mean(x.^2 ) - (mean(x))^2; X{6,18} = mean(y.^2 ) - (mean(y))^2;
X{6,19} = mean(x.*y ) - mean(x)*mean(y);
X{6,20} = s;
X{9,11} = r; X{9,12} = r^2;
X{9,14} = m;
X{9,17} = dm;
xlswrite([filename,'.xls'],X); clear X;

%Also save a simple two column Excel file, for application of linear
%regression from scratch
X{1,1} = 'x'; X{1,2} = 'y';
for n=1:length(x)
    X{1+n,1} = x(n);
    X{1+n,2} = y(n);
end
filename = ['bestfit y=mx test data ',strrep(datestr(now),':','-')];
xlswrite([filename,'.xls'],X);    

%

%Plot sum of squared deviations vs m
N = 1000;
m = linspace( -2*abs(m), 2*abs(m), N ); SSD = zeros(1,N);
for n=1:N
    SSD(n) = sum( (y - m(n)*x).^2 );
end
imin = find( SSD==min(SSD) ); imin = imin(1); 
SSDmin = SSD(imin); mmin = m(imin);
plot(m,SSD); hold on; plot(mmin,SSDmin,'r*');
axis tight; grid on; set(gca,'fontsize',fsize);
xlabel('m','fontsize',fsize); 
ylabel('Sum of squared differences (SSD)','fontsize',fsize);
title(['SSD from y = mx. m=',num2str(mmin)],'fontsize',fsize);
print( gcf, [filename,' SSD vs m'],'-dpng','-r300'); close(gcf);

%%

%Line of best fit function yfit = m*x, with product moment correlation
%coefficient r
function [yfit,xfit,r,m,dm,yupper,ylower,s] = bestfit(x,y)

%Find any x or y values that are NaN or Inf
ignore = isnan(abs(x)) | isnan(abs(y)) | isinf(abs(x)) | isinf(abs(y));
x(ignore) = []; y(ignore) = [];
n = length(x);

%Compute line of best fit
xbar = mean(x); ybar = mean(y); xybar = mean(x.*y);
xxbar = mean(x.^2 ); yybar = mean(y.^2 );
Vx = xxbar - xbar^2; Vy = yybar - ybar^2;
COVxy = xybar - xbar*ybar;
m = xybar/xxbar; r = COVxy/sqrt( Vx*Vy );
[x,i] = sort(x); y = y(i);
yfit = m*x; xfit = x;

%Compute error in gradient m
s = sqrt( (1/(n-1))*sum( (y - yfit).^2 ) );
dm = s/sqrt(n*Vx);

%Determine envelope lines
yupper = (m+dm)*x;
ylower = (m-dm)*x;

%End of code