% Line of Best Fit (linear regression) demonstration for
% lines of the form y = mx + c.
%
% This code generates N (x,y) samples based upon a positive or negative
% correlation + some random noise. A least-squares regression is then
% performed and graphs and Excel sheet(s) are generated.
%
% It is not pre-assumed that y= 0 when x = 0 i.e. for y = mx and direct
% proportion. Equal statistical weighting is given to the determination of
% line parameters m and c.
function y_mx_c_fit_simulation

%% INPUTS %%

%Create random data based upon average line y = m*x + c
N = 100; m = -4; c = 5;
xmin = 0; xmax = 10;

%Characterize normally distributed deviations from (x,y)
sigma_x = 5; sigma_y = 5;

%Fontsize for graphs
fsize = 18;

%%

%Create data vectors
x = linspace( xmin, xmax, N ) + sigma_x*randn(1,N);
y = m*x + c + sigma_y*randn(1,N);

%Determine line of best fit to x,y data
[yfit,xfit,r,m,c,dm,dc,yupper,ylower,s] = bestfitc(x,y);

%Determine filename
filename = ['bestfit y=mx+c simulation ',strrep(datestr(now),':','-')];

%Plot line of best fit, overlaid with data
figure('name','line of best fit','color',[1 1 1]);
plot( xfit, yfit, 'k-', xfit, yupper, 'b--', xfit, ylower, 'g--' );
hold on; plot( x,y,'r+');
xlabel('x','fontsize',fsize); ylabel('y','fontsize',fsize);
set(gca,'fontsize',fsize); grid on;
if c >=0
    c_sign = '+';
else
    c_sign = '-';
end
title( {['Line of best fit y = ',num2str(m,3),'x ',c_sign,' ',num2str(abs(c),3)],...
    ['\Deltam = ',num2str(dm,3),', \Deltac = ',num2str(dc,2),', r = ',num2str(r,3)] }, 'fontsize',fsize );
print( gcf, [filename,'.png'],'-dpng','-r300' );
close(gcf);

%Plot line of best fit surface
line_of_best_fit_surface( x,y,m,c,N,fsize,filename )

%Create Excel sheet
X{1,1} = ['LINE OF BEST FIT y = mx+c DEMONSTRATION. Auto generated at ',datestr(now)];
X{2,12} = 'N'; X{2,13} = N;
X{5,1} = 'x'; X{5,2} = 'y'; X{5,3} = 'x^2'; X{5,4} = 'y^2'; X{5,5} = 'xy';
X{5,6} = 'xfit'; X{5,7} = 'yfit'; X{5,8} = 'ylower'; X{5,9} = 'yupper';
X{5,11} = 'xbar'; X{5,12} = 'ybar'; X{5,13} = 'x^2 bar'; X{5,14} = 'y^2 bar'; X{5,15} = 'xy bar';
X{5,17} = 'Vx'; X{5,18} = 'Vy'; X{5,19} = 'Cov[x,y]'; X{5,20} = 's';
X{8,11} = 'r'; X{8,12} = 'r^2';  X{8,14} = 'm'; X{8,15} = 'c';  X{8,17} = 'dm';  X{8,18} = 'dc';
for n=1:length(x)
    X{5+n,1} = x(n);
    X{5+n,2} = y(n);
    X{5+n,3} = x(n)^2;
    X{5+n,4} = y(n)^2;
    X{5+n,5} = x(n)*y(n);
    X{5+n,6} = xfit(n);
    X{5+n,7} = yfit(n);
    X{5+n,8} = ylower(n);
    X{5+n,9} = yupper(n);
end
X{6,11} = mean(x);  X{6,12} = mean(y); X{6,13} = mean(x.^2);
X{6,14} = mean(y.^2); X{6,15} = mean(y.*x);
X{6,17} = mean(x.^2 ) - (mean(x))^2; X{6,18} = mean(y.^2 ) - (mean(y))^2;
X{6,19} = mean(x.*y ) - mean(x)*mean(y);
X{6,20} = s;
X{9,11} = r; X{9,12} = r^2;
X{9,14} = m;
X{9,15} = c;
X{9,17} = dm;
X{9,18} = dc;
xlswrite([filename,'.xls'],X);

%Also save a simple two column Excel file, for application of linear
%regression from scratch
X{1,1} = 'x'; X{1,2} = 'y';
for n=1:length(x)
    X{1+n,1} = x(n);
    X{1+n,2} = y(n);
end
filename = ['bestfit y=mx+c test data ',strrep(datestr(now),':','-')];
xlswrite([filename,'.xls'],X);    

%%

%Line of best fit function yfit = m*x +c, with product moment correlation
%coefficient r
function [yfit,xfit,r,m,c,dm,dc,yupper,ylower,s] = bestfitc(x,y)

%Find any x or y values that are NaN or Inf
ignore = isnan(abs(x)) | isnan(abs(y)) | isinf(abs(x)) | isinf(abs(y));
x(ignore) = []; y(ignore) = [];
n = length(x);

%Compute line of best fit
xbar = mean(x); ybar = mean(y); xybar = mean(x.*y);
xxbar = mean(x.^2 ); yybar = mean(y.^2 );
Vx = xxbar - xbar^2; Vy = yybar - ybar^2;
COVxy = xybar - xbar*ybar;
m = COVxy/Vx; c = ybar - m*xbar; r = COVxy/sqrt( Vx*Vy );
[x,i] = sort(x); y = y(i);
yfit = m*x + c; xfit = x;

%Compute errors in gradient m and intercept c
s = sqrt( (1/(n-2))*sum( (y - yfit).^2 ) );
dm = s/sqrt(n*Vx);
dc = ( s/sqrt(n) )*sqrt( 1 + (xbar^2)/Vx );

%Determine envelope lines
yupper = (m+dm)*x + c - dc;
ylower = (m-dm)*x + c + dc;

%%

%Plot of best fit surface
function line_of_best_fit_surface( x,y,m,c,NN,fsize,filename )

%Compute line of best fit coefficients from minima of surface of sum of
%square y deviations
a = linspace( -2*abs(c), 2*abs(c), NN );
b = linspace( -2*abs(m), 2*abs(m), NN );
[a,b] = meshgrid(a,b);
S = zeros(NN,NN);
for n=1:length(x)
    S = S + ( a + b*x(n) - y(n) ).^2 ;
end

%Determine indices of array S which correspond to global minima
[i,j] = find(S==min(min(S)));

%Plot surface of square deviations from line of best fit vs best fit
%parameters
S = -log(S);
figure('color',[1 1 1],'renderer','opengl');
s = surf(a,b,S,S);
hold on;
plot3( a(i,j),b(i,j),S(i,j),'m*','markersize',10)
axis tight
shading interp
interp_colormap(500);
xlabel('c','fontsize',fsize), ylabel('m','fontsize',fsize)
zlabel('-log(S)','fontsize',fsize)
title({'-log( Sum of (y - mx - c)^2 )',...
    ['m = ',num2str(m,3),', c = ',num2str(c,3)] },'fontsize',fsize)
set(gca,'fontsize',fsize);
colorbar('fontsize',fsize);
view(3)
print(gcf,'-dpng','-r300',['3D surface ',filename,'.png'])
view(2);
print(gcf,'-dpng','-r300',['2D surface ',filename,'.png'])
close(gcf);

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code